package foliocrawl;

import folios.FolioDataHelper;
import folios.FolioIteratorCallback;

import folios.FolioManager;

import intradoc.common.ServiceException;

import intradoc.data.DataBinder;

import intradoc.data.DataException;

import intradoc.data.DataResultSet;

import java.util.Map;
import java.util.Vector;

/***
 * This is the folio iterator callback. It is instantiated by the Folio Crawl Service Handler,
 * although it is called by the Folio Iterator class.
 ***/
public class FolioCrawlIterator implements FolioIteratorCallback {
  DataResultSet folioResultSet;

  public void init(DataBinder folio, FolioManager manager, Map options) throws DataException, ServiceException
  {
    folioResultSet = (DataResultSet)options.get("folioResultSet");
  }

  public void beginNode(int depth, String id, String type, DataBinder folio)
      throws DataException, ServiceException
  {
    Map props = FolioDataHelper.getNodeProperties(folio, id);
    String docName = (String)props.get("xcsd:dDocName");
    // System.out.println("... in Folio Crawl Iterator - begin node: id: "+id+" depth: "+Integer.toString(depth)+" type: "+type);
    Vector row = folioResultSet.createEmptyRow();
    // the result set colums are "id", "type", "level", "title", "description", "dDocTitle", "dDocName", "dRevLabel", "dID",
    //                            0     1       2        3        4              5            6           7            8    
    //          "dDocType", "dDocAuthor", "docURL", "thumbnailURL", "dFormat"
    //          9           10            11        12              13
    row.setElementAt(id, 0);
    row.setElementAt(type, 1);
    // the root is level 1 and its children are level 2... so shift them to 0 and 1
    row.setElementAt(Integer.toString(depth-1), 2);
    row.setElementAt(props.get("xcst:name"), 3);
    String temp = (String) props.get("xcst:description");
    row.setElementAt(temp,4);
    if(type.equalsIgnoreCase("node")) {
      // no extra props for node
    }
    else if(type.equalsIgnoreCase("slot")) {
      // check if this is an external link
      temp = (String) props.get("xcst:isExternalItem");
      if(temp != null && temp.length() > 0) {
        // its a link. reset the type
        row.setElementAt(props.get("xcst:externalType"), 1);
        row.setElementAt(props.get("xcst:externalTarget"), 11);
      }
      else {
        // check if the slot is empty
        if(docName != null && docName.length() > 0) {
          row.setElementAt(props.get("xcsd:dDocTitle"), 5);
          row.setElementAt(docName,6);
          row.setElementAt(props.get("xcsd:dRevLabel"), 7);
          row.setElementAt(props.get("xcsd:dID"), 8);
          row.setElementAt(props.get("xcsd:dDocType"), 9);
          row.setElementAt(props.get("xcsd:dDocAuthor"), 10);
          row.setElementAt(props.get("xcsd:docURL"), 11);
          temp = (String)props.get("xcsd:dRendition1");
          if(temp.equalsIgnoreCase("T")) {
            // calculate the thumbnail url
            temp = (String)props.get("xcsd:docURL");
            temp = temp.substring(0,temp.lastIndexOf("."));
            System.out.println("... Folio Iterator, temp is "+temp);
            temp = temp + "@T~"+(String)props.get("xcsd:dRevLabel")+".jpg";
            System.out.println("... Folio Iterator, temp is now "+temp);
            row.setElementAt(temp,12);
          }
          else {
            // use the doctype
            temp = (String)props.get("xcsd:dDocType");
            row.setElementAt("/idc/images/docgifs/"+temp.toLowerCase()+".gif",12);
          }
          row.setElementAt(props.get("xcsd:dFormat"), 13);
        }
      }
    }
    folioResultSet.addRow(row);
  }

  public void endNode(int depth, String id, String type, DataBinder folio)
      throws DataException, ServiceException
  {
    // no processing required.
  }

  public void finish() throws DataException, ServiceException
  {
    // System.out.println("... in Folio Crawl Iterator - at end of Folio crawl, nodes found: "+folioResultSet.getNumRows());
  }

  public static Object idcVersionInfo(Object arg)
  {
    return "releaseInfo=dev,releaseRevision=$Rev$";
  }
}
